/*
    9 - 10, 14 and 26 Feb 2013

    Randomly display links to a few pages on the site -
    extracted from cgi-bin/dstqview/destaqLL.htm
    (LL = ISO language code), generated by destaq (see
    destaq.c), called by shell script destaq (manually
    or via crontab).

    Copyright 2013-2014, R.Harmsen, all right reserved

    26 Feb 2013: variable number of entries shown.
    24 May 2014: Texts for Interlingua in addition to English and Dutch.
    26 May 2014: If no language specified, randomise between
       Dutch, English, Interlingua, Portuguese and German
      (35, 35, 20, 5 and 5% of cases on average).
     5 Oct 2014: More elegant solution for multilingual texts
    24 December 2015: Changed percentages from
       50 nl, 20 en, 15 ia, 5 eo, 5 pt and 5 de
       to
       35 nl, 25 en, 25 ia, 5 eo, 5 pt and 5 de.
    20 October 2019: ISO-8859-1 will be UTF-8 from now on, because all
       HTML has been converted.
    10 May 2021: removed last vestiges of Adsense code. Was
       already disabled since 2012.
    5 February 2023: Better CSS mechanism.
*/

#include <stdio.h>
#include <string.h>
#if !defined(_Windows) && !defined (__MSDOS__)
   #include <sys/types.h>
   #include <unistd.h>
#endif
#include <stdlib.h>
#include <sys/stat.h>

#include "../cgitools/cgitools.h"
#include "../fstrfind/fstrfind.h"

static struct progdata_s
{
   char lang[2+1];
   int LangIndex;
   int randomlang;
   int numentries;
   int navifooter;
   char *scriptname;
} progdata;

static void invocpar (struct progdata_s *p_progdata);
static void htmlhead (struct progdata_s *p_progdata);
static void showform (struct progdata_s *p_progdata);
static void htmlfoot (struct progdata_s *p_progdata);

#define MAXDISPLAY 1500
#define TRIESMORE  4
#define EstimatedEntrySize 300

#if defined(_Windows) || defined (__MSDOS__)
  #define DRAND() ((double)rand()   / (double)RAND_MAX)
  #define SRAND() (srand  ((unsigned)time(NULL)))
#else
  #define DRAND() ((double)random() / (double)0x7fffffffL)
  #define SRAND() (srandom((unsigned)time(NULL) + getpid()))
#endif

int main (int argc, char **argv)
{
   progdata.scriptname = "../cgi-bin/dstqview.cgi";

   invocpar(&progdata);
   htmlhead(&progdata);
   showform(&progdata);
   htmlfoot(&progdata);

   return 0;
}

static void invocpar (struct progdata_s *p_progdata)
{
   char *query;
   char **Vars = NULL, **V;

   p_progdata->numentries = 3;
   p_progdata->navifooter = 1; /* == yes */

   query = getenv("QUERY_STRING");
   if (query)
   {
      Vars = CgiConvVars(query, NULL);

      for (V = Vars; V[0]; V += 2)
      {
         if (strcmp(V[0], "lang") == 0 && strlen(V[1]))
         {
            sprintf(p_progdata->lang, "%2.2s", V[1]);
         }
         if (strcmp(V[0], "num") == 0 && strlen(V[1]))
         {
            p_progdata->numentries = atoi(V[1]);
            if (p_progdata->numentries < 3)
                p_progdata->numentries = 3;
            else if (p_progdata->numentries > MAXDISPLAY)
                p_progdata->numentries =  MAXDISPLAY;
         }
         if (strcmp(V[0], "navifooter") == 0 && strlen(V[1]))
         {
            if (strcmp(V[1], "no") == 0)
               p_progdata->navifooter = 0;
         }
      }
   }

   SRAND();
   p_progdata->randomlang = 0;
   if (!p_progdata->lang[0])
   {
      double ranval = DRAND();
      double cumul = 0.0;

      p_progdata->randomlang = 1;
      if (ranval <= (cumul += 0.35))
         strcpy(p_progdata->lang, "nl");
      else if (ranval <= (cumul += 0.25))
         strcpy(p_progdata->lang, "en");
      else if (ranval <= (cumul += 0.25))
         strcpy(p_progdata->lang, "ia");
      else if (ranval <= (cumul += 0.05))
         strcpy(p_progdata->lang, "eo");
      else if (ranval <= (cumul += 0.05))
         strcpy(p_progdata->lang, "pt");
      else
         strcpy(p_progdata->lang, "de");
   }
}

static char HeadText1[] =
   "<!doctype html>\n"
   "<html lang=en>\n"
   "<head>\n"
   "<meta name=viewport content=\"width=device-width, initial-scale=1\">\n"
/* 20 October 2019: ISO-8859-1 will be UTF-8 from now on, because all HTML has been converted */
   "<meta http-equiv=\"Content-Type\" CONTENT=\"text/html; charset=UTF-8\">\n"
   "<meta name=\"robots\" content=\"noindex, nofollow\">\n"
   "<link rel=\"stylesheet\" href=\"../css/dstqview.css\">\n"
   "<link rel=\"stylesheet\" href=\"../cgi-bin/colschem.cgi\">\n"
   "<title>"
   ;

static struct TitleStruct
{
   char Lang[3];
   char TitleText[120];
} TitleTab[] =
{
   { "nl", "Geselecteerde artikelen (%s: taal is Nederlands)" },
   { "ia", "Articulos selecte (%s: lingua es interlingua)" },
   { "en", "Featured articles (%s: language is English)" },
   { "de", "Ausgew&auml;hlte Artikel (%s: Sprache ist Deutsch)" },
   { "pt", "Art&iacute;culos em destaque (%s: l&iacute;ngua &eacute; portugu&ecirc;s)" },
   { "  ", "Articulos selecte (lingua es %s)" },
};

static char HeadText2[] =
   "</title>\n"
   ;

static char HeadText3[] =
   "</title>\n"
   "</head>\n"
   "<body>\n\n"
   ;

static struct NoveSelectionStr
{
   char Text[30];
} NovSelTab[] =
{
   { "Nieuwe selectie" },
   { "Nove selection" },
   { "New selection" },
   { "Neue Auswahl" },
   { "Selec&ccedil;&atilde;o nova" },
   { "Nove selection" },
};

static void htmlhead (struct progdata_s *p_progdata)
{
   for (p_progdata->LangIndex = 0;
        p_progdata->LangIndex < sizeof TitleTab / sizeof TitleTab[0];
        p_progdata->LangIndex++)
   {
      if (strcmp(p_progdata->lang, TitleTab[p_progdata->LangIndex].Lang) == 0 ||
          TitleTab[p_progdata->LangIndex].Lang[0] == ' ')
      {
         break;
      }
   }

   printf("Content-type: text/html\n\n");
   printf("%s", HeadText1);
   printf(TitleTab[p_progdata->LangIndex].TitleText, p_progdata->lang);
   printf("%s\n", HeadText2);
   if (!p_progdata->navifooter)
      printf("<style>body {margin: 10px;}</style>\n");
   printf("%s\n", HeadText3);

   if (p_progdata->navifooter)
      printf("<div class=rh-article-noad>\n\n");

   printf("<h1>");
   printf(TitleTab[p_progdata->LangIndex].TitleText, p_progdata->lang);
   printf("</h1>\n");

   fflush(stdout);
}

static void htmlfoot (struct progdata_s *p_progdata)
{
   if (p_progdata->navifooter)
   {
      printf("</div>\n<div class=rh-navifooter-noad>\n<br>\n\n");

      printf("<form action=\"%s\?lang=%s&amp;num=%d\" method=post>\n",
         p_progdata->scriptname,
         p_progdata->randomlang ? "" : p_progdata->lang,
         p_progdata->numentries);

      printf(
         "<input type=Submit Name=Randomise value=\"%s\"> \n"
         "</form>\n\n", NovSelTab[p_progdata->LangIndex].Text);
   }

   if (strcmp(p_progdata->lang, "nl") == 0)
   {
      printf("<p class=pagelink>"
         "<a target=_top href=\"../index/viewview.htm\">Bekeken pagina&rsquo;s</a><br>\n"
         "<a target=_top href=\"../index/menu.htm\">Lang menu</a><br>\n"
         "<a target=_top href=\"../accessng.htm\">Kort menu</a><br>\n"
         "<a target=_top href=\"../index.htm\">Beginpunt van deze site</a>\n"
         "</p>\n");

      printf("<p class=NCol>\n"
         "<a class=NCol href=\"../index/colexpnl.htm\">Kleuren:</a>\n"
         "<a class=NCol href=\"../cgi-bin/colcooky.cgi?NoWeirdColours=1\">Neutraal</a>\n"
         "<a class=NCol href=\"../cgi-bin/colcooky.cgi?NoWeirdColours=0\">Raar</a>\n"
         "<a class=NCol href=\"../cgi-bin/colcooky.cgi?NoWeirdColours=R\">Geen voorkeur</a>\n"
         "<a class=NCol href=\"javascript:location.reload();\">Pagina opnieuw laden</a>\n"
         "</p>\n\n");

   }
   else if (strcmp(p_progdata->lang, "ia") == 0)
   {
      printf("<p class=pagelink>"
         "<a target=_top href=\"../index/viewview.htm\">Un vista sur vistas</a><br>\n"
         "<a target=_top href=\"../index/menu.htm\">Menu longe</a><br>\n"
         "<a target=_top href=\"../accessng.htm\">Menu curte</a><br>\n"
         "<a target=_top href=\"../index.htm\">Porta de entrata de iste sito web</a>\n"
         "</p>\n");

      printf("<p class=NCol>\n"
         "<a class=NCol href=\"../index/colexpen.htm\">Colores:</a>\n"
         "<a class=NCol href=\"../cgi-bin/colcooky.cgi?NoWeirdColours=1\">Enoiose</a>\n"
         "<a class=NCol href=\"../cgi-bin/colcooky.cgi?NoWeirdColours=0\">Extravagante</a>\n"
         "<a class=NCol href=\"../cgi-bin/colcooky.cgi?NoWeirdColours=R\">Necun preferentia</a>\n"
         "<a class=NCol href=\"javascript:location.reload();\">Recarga le pagina</a>\n"
         "</p>\n\n");
   }
   else
   {
      printf("<p class=pagelink>"
         "<a target=_top href=\"../index/viewview.htm\">A view on views</a><br>\n"
         "<a target=_top href=\"../index/menu.htm\">Long menu</a><br>\n"
         "<a target=_top href=\"../accessng.htm\">Short menu</a><br>\n"
         "<a target=_top href=\"../index.htm\">This site&rsquo;s entry point</a>\n"
         "</p>\n");

      printf("<p class=NCol>\n"
         "<a class=NCol href=\"../index/colexpen.htm\">Colours:</a>\n"
         "<a class=NCol href=\"../cgi-bin/colcooky.cgi?NoWeirdColours=1\">Neutral</a>\n"
         "<a class=NCol href=\"../cgi-bin/colcooky.cgi?NoWeirdColours=0\">Weird</a>\n"
         "<a class=NCol href=\"../cgi-bin/colcooky.cgi?NoWeirdColours=R\">No preference</a>\n"
         "<a class=NCol href=\"javascript:location.reload();\">Reload screen</a>\n"
         "</p>\n\n");
   }

   if (p_progdata->navifooter)
   {
      printf("\n</div>\n\n");
   }

   printf("</body>\n</html>\n");
   fflush(stdout);
}

static char DestaqFilesPath[] = "dstqview/destaq%2.2s.htm";
static void showform (struct progdata_s *p_progdata)
{
   int i, j, k;
   int duplicate;
   FILE *fp;
   struct stat st;
   char DestaqFile[sizeof DestaqFilesPath];
   long *AlreadyShown;

   AlreadyShown = malloc(p_progdata->numentries * (sizeof *AlreadyShown));
   if (!AlreadyShown)
   {
      printf("<p>Cannot malloc %d elements for \'Already Shown\' table\n", p_progdata->numentries);
      return;
   }

   sprintf(DestaqFile, DestaqFilesPath, p_progdata->lang);

   if (stat(DestaqFile, &st))
   {
      // printf("<p>Error: cannot stat() file<br>%s</p>\n", DestaqFile);
      // Prefer a less technical, more visitor oriented error message:

      printf("<p>No highlighted fragments were marked for language &lsquo;%s&rsquo;</p>\n",
         p_progdata->lang);
      free(AlreadyShown);

      return;
   }

   #if defined(_Windows) || defined (__MSDOS__)
      fp = fopen(DestaqFile, "rb");
   #else
      fp = fopen(DestaqFile, "r");
   #endif
   if (!fp)
   {
      printf("<p>Error: cannot fopen() file<br>%s</p>\n", DestaqFile);
      free(AlreadyShown);

      return;
   }

   printf("<FORM ACTION=\"javascript:location.reload();\" METHOD=POST>\n");
   printf("<INPUT TYPE=Submit Name=Refresh VALUE=\"Refresh (F5)\">\n");
   printf("</FORM>\n\n");

#if 0
   /* See below */
   printf("<FORM ACTION=\"%s\?lang=%s&amp;num=%d\" METHOD=POST>\n",
      p_progdata->scriptname,
      p_progdata->randomlang ? "" : p_progdata->lang,
      p_progdata->numentries);

   printf(
      "<INPUT TYPE=Submit Name=Randomise VALUE=\"%s\"> \n"
      "</FORM>\n\n", NovSelTab[p_progdata->LangIndex].Text);
#endif

   for (k = 0; k < p_progdata->numentries; k++)
   {
       AlreadyShown[k] = -1;
   }

   // Already done earlier, don't repeat it here.
   // SRAND();

   for (i = 0, j = 0; i < p_progdata->numentries && j < p_progdata->numentries * TRIESMORE; i++, j++)
   {
      long pos1, pos2 = -1;
      int c;
      int FromFirstEntry = 0;

      pos1 = (long)((double)st.st_size * DRAND());
      /* This is interpreted as a position in the middle of
         an entry in the file. We now have to go back a little,
         so even the first entry has a chance of being randomly
         selected.
       */
      if (pos1 < EstimatedEntrySize)
      {
         pos1 = 0;
         FromFirstEntry++;
         fseek(fp, pos1, SEEK_SET);
      }
      else
      {
         pos1 -= EstimatedEntrySize;
         /* Set this random point as the starting position for
            the search. */
         fseek(fp, pos1, SEEK_SET);
         pos1 = fstrfindex(fp, "<hr>", 1, 1, 1);
      }

      if (pos1 >= 0)
      {
         pos2 = fstrfindex(fp, "<hr>", 1, 1, 1);
      }

      if (pos1 < 0 || pos2 < 0)
      {
         i--;
         continue;
      }

      /* Check if the random entry was already displayed before */
      duplicate = 0;
      for (k = 0; k < p_progdata->numentries; k++)
      {
          if (AlreadyShown[k] < 0)
          {
             AlreadyShown[k] = pos1;
             break;
          }
          else if (AlreadyShown[k] == pos1)
          {
             duplicate = 1;
             break;
          }
      }

      if (duplicate)
      {
         i--;
         continue;
      }

      /* Read and output the entry, from pos1 to pos2 */
      if (FromFirstEntry)
         printf("\n<hr>\n");

      fseek(fp, pos1, SEEK_SET);
      while (pos1++ < pos2)
      {
         c = getc(fp);
         if (c == EOF)
            break;
         fputc(c, stdout);
      }
   }

   printf("\n<hr>\n");

   fclose(fp);
   free(AlreadyShown);

   printf("<FORM ACTION=\"%s\?lang=%s&amp;num=%d\" METHOD=POST>\n",
      p_progdata->scriptname,
      p_progdata->randomlang ? "" : p_progdata->lang,
      p_progdata->numentries);

   printf(
      "<INPUT TYPE=Submit Name=Randomise VALUE=\"%s\"> \n"
      "</FORM>\n\n", NovSelTab[p_progdata->LangIndex].Text);
}
